#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const progress_1 = __importDefault(require("progress"));
const AWSS3Glacier_1 = require("./service/AWSS3Glacier");
const nullableReadFile_1 = require("./util/nullableReadFile");
const BackblazeB2_1 = require("./service/BackblazeB2");
const upload_1 = require("./upload/upload");
const minimist = require("minimist");
const DEFAULT_CONCURRENT_UPLOADS = 3;
const DEFAULT_MAX_RETRIES_PER_PART = 5;
const SERVICES = {
    aws: {
        service: AWSS3Glacier_1.AWSS3Glacier,
        options: AWSS3Glacier_1.parseAWSS3GlacierOptions,
    },
    b2: {
        service: BackblazeB2_1.BackblazeB2,
        options: BackblazeB2_1.parseBackblazeB2Options,
    },
};
const sessionPath = (workDir) => {
    return path_1.default.join(workDir, "session");
};
const statePath = (workDir, key) => {
    return path_1.default.join(workDir, `state_${key}`);
};
const main = (rawArgs, progressBar) => __awaiter(this, void 0, void 0, function* () {
    const args = minimist(rawArgs);
    const filePath = args.file;
    const fileStats = yield fs_1.promises.stat(filePath);
    if (!fileStats.isFile()) {
        throw new Error(`${filePath} is not a file`);
    }
    const workingDirectory = args.work;
    if (!(yield fs_1.promises.stat(workingDirectory)).isDirectory()) {
        throw new Error(`${workingDirectory} is not a directory`);
    }
    const concurrentUploads = +args.concurrency || DEFAULT_CONCURRENT_UPLOADS;
    const maximumRetriesPerPart = +args.retries || DEFAULT_MAX_RETRIES_PER_PART;
    const serviceName = args.service.toLowerCase();
    const { service, options: parseOptions } = SERVICES[serviceName];
    const serviceOptions = parseOptions(args);
    const serviceState = yield service.fromOptions(serviceOptions);
    const ctx = {
        concurrentUploads,
        maximumRetriesPerPart,
        file: {
            path: filePath,
            size: fileStats.size,
        },
        logError: (msg) => {
            progressBar.interrupt(msg);
        },
        readState: (key) => {
            return nullableReadFile_1.nullableReadFile(statePath(workingDirectory, key));
        },
        resumeSession: () => {
            return nullableReadFile_1.nullableReadJson(sessionPath(workingDirectory));
        },
        updateProgress: (p) => {
            progressBar.update(p.completeRatio, {
                title: p.description,
            });
        },
        writeSession: (session) => {
            return fs_1.promises.writeFile(sessionPath(workingDirectory), JSON.stringify(session));
        },
        writeState: (key, value) => {
            return fs_1.promises.writeFile(statePath(workingDirectory, key), value);
        },
    };
    yield upload_1.upload(ctx, service, serviceState);
});
const progressBar = new progress_1.default(":title [:bar] :percent", {
    total: 100,
    complete: "=",
    incomplete: " ",
    renderThrottle: 0,
    clear: true,
});
main(process.argv.slice(2), progressBar)
    .then(() => {
    progressBar.terminate();
    console.log(`File successfully uploaded`);
}, (err) => {
    progressBar.terminate();
    console.error(err);
});
//# sourceMappingURL=main.js.map