"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const filesize_1 = __importDefault(require("filesize"));
const p_queue_1 = __importDefault(require("p-queue"));
const state_1 = require("./state");
const session_1 = require("./session");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
exports.upload = (ctx, svc, s) => __awaiter(this, void 0, void 0, function* () {
    const partSize = Math.max(svc.minPartSize, Math.min(svc.maxPartSize, 
    // TODO Part size should be loaded from session
    // as idealPartSize can be non-deterministic.
    yield svc.idealPartSize(s, ctx.file.size)));
    const partsNeeded = Math.ceil(ctx.file.size / partSize);
    if (partsNeeded < 1) {
        throw new Error(`File is too small`);
    }
    if (partsNeeded > svc.maxParts) {
        throw new Error(`File is too big (${filesize_1.default(ctx.file.size)}) and cannot fit into ${svc.maxParts} ${filesize_1.default(svc.maxPartSize)} parts ` +
            `(requires ${partsNeeded} parts)`);
    }
    ctx.updateProgress({
        description: `Checking for existing work...`,
        completeRatio: 0,
    });
    // TODO Ensure that session used same file with same contents and part size.
    const uploadId = (yield session_1.maybeLoadExistingUploadId(ctx)) || (yield (() => __awaiter(this, void 0, void 0, function* () {
        ctx.updateProgress({
            description: `No existing upload found, initiating new upload...`,
            completeRatio: 0,
        });
        const uploadId = yield svc.initiateNewUpload(s, path_1.default.basename(ctx.file.path), partSize);
        yield session_1.saveUploadIdToSession(ctx, uploadId);
        return uploadId;
    }))());
    // TODO Add ability to verify integrity of hashes.
    const partHashes = yield Promise.all(new Array(partsNeeded)
        .fill(0)
        .map((_, part) => state_1.loadPartHash(ctx, part)));
    let partsCompleted = partHashes.filter(h => h != null).length;
    const retryCounts = {};
    const queue = new p_queue_1.default({ concurrency: ctx.concurrentUploads });
    const updateUploadProgress = () => {
        const partsRemaining = partsNeeded - partsCompleted;
        ctx.updateProgress({
            description: `Uploading ${partsRemaining} ${filesize_1.default(partSize)} parts (${filesize_1.default(partsRemaining * partSize)})...`,
            completeRatio: Math.min(0.99, partsCompleted / partsNeeded),
        });
    };
    updateUploadProgress();
    const queuePartUploadTask = (part) => {
        const start = part * partSize;
        const end = Math.min(ctx.file.size, (part + 1) * partSize) - 1;
        queue.add(() => svc.uploadPart(s, uploadId, highWaterMark => fs_1.default.createReadStream(ctx.file.path, { highWaterMark, start, end }), { number: part, start, end })
            .then(({ hash }) => handleUploadPartSuccess(part, hash), err => handleUploadPartFailure(part, err)));
    };
    const handleUploadPartSuccess = (part, hash) => {
        partsCompleted++;
        partHashes[part] = hash;
        state_1.savePartHash(ctx, part, hash);
        updateUploadProgress();
    };
    // TODO Abort queue on error and add option to allowing ignoring this session.
    const handleUploadPartFailure = (part, err) => {
        const r = retryCounts[part] || 0;
        if (r > ctx.maximumRetriesPerPart) {
            throw new Error(`Part ${part} failed to upload ${ctx.maximumRetriesPerPart} times, with final error: ${err}`);
        }
        retryCounts[part] = r + 1;
        ctx.logError(`Failed to upload part ${part} with error "${err}", retrying (${r})...`);
        queuePartUploadTask(part);
    };
    for (const [part, hash] of partHashes.entries()) {
        if (hash) {
            continue;
        }
        queuePartUploadTask(part);
    }
    // NOTE: The queue could change during uploads due to retry,
    // so it's not as simple as awaiting all initial upload Promises.
    yield queue.onIdle();
    ctx.updateProgress({
        description: "All parts have been uploaded, finalising...",
        completeRatio: 0.99,
    });
    yield svc.completeUpload(s, uploadId, ctx.file.size, partHashes);
});
//# sourceMappingURL=upload.js.map